/******************************************************************************
* C Source: Write Image                                                       *
*                                                                             *
* File:                                                                       *
*    writeimg.c                                                               *
*                                                                             *
* Description:                                                                *
*    This file contains functions for writing images to disk.                 *
*                                                                             *
* Module List:                                                                *
*    Write_FITS_Extracted_Image                                               *
*       Write an image to a FITS file.                                        *
*    Write_HHH_Extracted Image                                                *
*       Write an image in GEIS (hhh/hhd) format.                              *
*    writefits:                                                               *
*       Create FITS image file.                                               *
*    writehhh:                                                                *
*       Create set of GEIS image files.                                       *
*                                                                             *
* History:                                                                    *
*    Created    : 14-DEC-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
* Copyright (c) 1993, 1994, Association of Universities for Research in       *
* Astronomy, Inc.  All Rights Reserved.                                       *
* Produced under National Aeronautics and Space Administration grant          *
* NAG W-2166.                                                                 *
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* access function not prototyped in standard place */
#if defined __VMS
#  include <unixio.h>
#elif defined __UNIX
#  include <unixstd.h>
#else
#endif

#include "writeimg.h"

#include "config.h"
#include "extrctn.h"
#include "generic.h"
#include "header.h"
#include "image.h"
#include "qfile.h"
#include "writeimg.h"

extern void qclose     ();

/******************************************************************************
* Module:                                                                     *
*    Write_FITS_Extracted_Image                                               *
*                                                                             *
* File:                                                                       *
*    writeimg.c                                                               *
*                                                                             *
* Description:                                                                *
*    Write an extracted image to files in FITS. The file name will be the     *
*    root passed in by the user with extension .fits.                         *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Image                                                                 *
*          IMAGE_DATA, Reference                                              *
*             The image data to write.                                        *
*       Header                                                                *
*          HEADER                                                             *
*             Image header to write with the data.                            *
*       imageFileRoot                                                         *
*          STRING, Reference                                                  *
*             Root for FITS filename                                          *
*    Input/Output Arguments:                                                  *
*          none                                                               *
*    Output Arguments:                                                        *
*          none                                                               *
*                                                                             *
* History:                                                                    *
*    Created    : 10-SEP-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/******************************************
* Write_FITS_Extracted_Image declarations *
********************************************/
void       Write_FITS_Extracted_Image( Image, Header, imageFileRoot )
IMAGE_DATA Image        ;
HEADER     Header       ;
STRING     imageFileRoot;
{
   char File_Name[ 14 ];
   int  Write_Status   ;

/***************************************************
* Write_FITS_Extracted_Image begin executable code *
***************************************************/
   strcpy( File_Name, imageFileRoot );
   strcat( File_Name, ".fits"       );

   Write_Status = writefits( Header, Image, File_Name );

   if ( Write_Status != 0 )
      fprintf( stderr, "\nError creating FITS file for %s.\n", imageFileRoot);

   else
      printf( "%s created.\n", File_Name );

/************************************
* Write_FITS_Extracted_Image page 2 *
************************************/

   return;
}
/*************************************************
* Write_FITS_Extracted_Image end executable code *
*************************************************/

/******************************************************************************
* Module:                                                                     *
*    Write_HHH_Extracted_Image                                                *
*                                                                             *
* File:                                                                       *
*    writeimg.c                                                               *
*                                                                             *
* Description:                                                                *
*    Write an extracted image to files in hhh/hhd format. The header and data *
*    file names will the root passed in by the user with extensions           *
*    .hhh/.hhd.                                                               *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Image                                                                 *
*          IMAGE_DATA, Reference                                              *
*             The image data to write.                                        *
*       Header                                                                *
*          HEADER                                                             *
*             Image header to write with the data.                            *
*       imageFileRoot                                                         *
*          STRING, Reference                                                  *
*             Root for FITS filename                                          *
*    Input/Output Arguments:                                                  *
*          none                                                               *
*    Output Arguments:                                                        *
*          none                                                               *
*                                                                             *
* History:                                                                    *
*    Created    : 10-SEP-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*****************************************
* Write_HHH_Extracted_Image declarations *
*****************************************/
void       Write_HHH_Extracted_Image( Image, Header, imageFileRoot )
IMAGE_DATA Image        ;
HEADER     Header       ;
STRING     imageFileRoot;
{
   char Header_File_Name[ 13 ];
   int  Write_Status          ;

/**************************************************
* Write_HHH_Extracted_Image begin executable code *
**************************************************/
   strcpy( Header_File_Name, imageFileRoot );
   strcat( Header_File_Name, ".hhh"        );

   Write_Status = writehhh( Header, Image, Header_File_Name );

   if ( Write_Status != 0 )
      fprintf( stderr, "\nError creating GEIS file for %s.\n", imageFileRoot);
   else
      printf( "%s/hhd created.\n", Header_File_Name );

/***********************************
* Write_HHH_Extracted_Image page 2 *
***********************************/

   return;
}
/************************************************
* Write_HHH_Extracted_Image end executable code *
************************************************/

/******************************************************************************
* Module:                                                                     *
*    Write_HHH_Header                                                         *
*                                                                             *
* File:                                                                       *
*    writeimg.c                                                               *
*                                                                             *
* Description:                                                                *
*    Write an hhh header file for an image.  The file name will the root      *
*       passed in by the user with extension .hhh.                            *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Header                                                                *
*          HEADER                                                             *
*             Image header to write with the data.                            *
*       imageFileRoot                                                         *
*          STRING, Reference                                                  *
*             Root for FITS filename                                          *
*    Input/Output Arguments:                                                  *
*          none                                                               *
*    Output Arguments:                                                        *
*          none                                                               *
*                                                                             *
* History:                                                                    *
*    Created    : 07-DEC-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/********************************
* Write_HHH_Header declarations *
********************************/
void       Write_HHH_Header(Header, imageFileRoot)
HEADER     Header       ;
STRING     imageFileRoot;
{
   char    Header_File_Name[ 13 ]; /* File name for header          */
   QFILE * ihead                 ; /* Pointer to opened header file */

   int i;

/*****************************************
* Write_HHH_Header begin executable code *
*****************************************/
   strcpy( Header_File_Name, imageFileRoot );
   strcat( Header_File_Name, ".hhh"        );

    /*
     * write header file: 80 byte records, CR carriage control
     */
    ihead = qcreat(Header_File_Name,80,1);
    for (i=0; i < Header->nhlines; i++) qwrite(ihead,Header->hlines[i],80);

    qclose(ihead);
    printf( "%s created.\n", Header_File_Name );


/**************************
* Write_HHH_Header page 2 *
**************************/

   return;
}
/***************************************
* Write_HHH_Header end executable code *
***************************************/

/******************************************************************************
* Module:                                                                     *
*    writefits                                                                *
*                                                                             *
* File:                                                                       *
*    writeimg.c                                                               *
*                                                                             *
* Description:                                                                *
*       Write FITS format data file                                           *
*                                                                             *
* Return Value:                                                               *
*    int: 0 if successful, -1 if fails.                                       *
*                                                                             *
* Arguments:                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       h                                                                     *
*          HEADER, Reference                                                  *
*             Header structure, contains proper image size updated with       *
*             fillhdr.                                                        *
*       image                                                                 *
*          short int, Reference                                               *
*             The image data to write.                                        *
*       headfile                                                              *
*          char, Reference                                                    *
*             Name of the hhh header file. Must end in .hhh. The final h is   *
*             changed to a d for the data file.                               *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 31-JUL-91 by R. White,                                      *
*    Delivered  : 16-JUN-92 by R. White,                                      *
*        Modified to write OpenVMS fixed-length record.                       *
*                                                                             *
*    Updated    : 10-SEP-93 by J. Doggett,                                    *
*        Adopted into software used to facilitate the accessing of data from  *
*            the CD-ROM set of the Digitized Sky Survey.                      *
*    Updated    : 06-JAN-94 by J. Doggett,                                    *
*       Changed #if defs to check for UNIX rather not OpenVMS when checking   *
*          if we might be writing over a file.                                *
*    Updated    : 10-JAN-94 by J. Doggett,                                    *
*       Modified #if def to abort if we're not on a known operating system.   *
*    Redelivered: 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
extern int
writefits(h,image,filename)
HEADER h;
short image[];
char *filename;
{
int i, npix;
QFILE *ifile;
char blank[80];

    /*
     * Check to see whether file already exists
     * (Don't bother under OpenVMS, just make a new version)
     */
    if (access(filename,0) == 0) {
#if defined __VMS

/*******************
* writefits page 2 *
*******************/

/* We'll just create a new version on OpenVMS systems */

#elif defined _UNIX
        fprintf(stderr, "Error: file %s already exists\n",filename);
        return(-1);

#else
   fprintf(stderr,"Unknown file system error in writefits");
   exit(EXIT_FAILURE);

#endif
    }

    /*
     * open file: 2880 byte records, no carriage control
     */
    ifile = qcreat(filename, 2880, 0);
    /*
     * write header
     */
    for (i=0; i < h->nhlines; i++) qwrite(ifile, h->hlines[i], 80);
    /*
     * write some blank lines if number of header lines is not a
     * multiple of 36 (2880 bytes)
     */
    for (i = 0; i<80; i++) blank[i] = ' ';
    for (i = ((h->nhlines-1) % 36) + 1; i<36; i++) qwrite(ifile, blank, 80);
    /*
     * write data one byte at a time to ensure proper byte order
     */
    npix = h->section_x_length * h->section_y_length;
    for (i=0; i<npix; i++) {
        qputc( (image[i]>>8) & 0xff, ifile);
        qputc(  image[i]     & 0xff, ifile);
    }
    /*
     * write pad to bring data up to multiple of 2880 bytes too
     */
    for (i = ((2*npix-1) % 2880) + 1; i<2880; i++) qputc(0,ifile);
    qclose(ifile);
    return(0);

} /* writefits */

/******************************************************************************
* Module:                                                                     *
*    writehhh                                                                 *
*                                                                             *
* File:                                                                       *
*    writeimg.c                                                               *
*                                                                             *
* Description:                                                                *
*       Write hhh/hhd format data files.                                      *
*                                                                             *
* Return Value:                                                               *
*    int: 0 if successful, -1 if fails.                                       *
*                                                                             *
* Arguments:                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       h                                                                     *
*          HEADER, Reference                                                  *
*             Header structure, contains proper image size updated with       *
*             fillhdr.                                                        *
*       image                                                                 *
*          short int, Reference                                               *
*             The image data to write.                                        *
*       headfile                                                              *
*          char, Reference                                                    *
*             Name of the hhh header file. Must end in .hhh. The final h is   *
*             changed to a d for the data file.                               *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 31-JUL-91 by R. White,                                      *
*    Delivered  : 16-JUN-92 by R. White,                                      *
*                  Modified to write OpenVMS fixed-length record.             *
*                                                                             *
*    Updated    : 10-SEP-93 by J. Doggett,                                    *
*        Adopted into software used to facilitate the accessing of data from  *
*            the CD-ROM set of the Digitized Sky Survey.                      *
*    Updated    : 06-JAN-94 by J. Doggett,                                    *
*       Changed #if defs to check for UNIX rather not OpenVMS when checking   *
*          if we might be writing over a file.                                *
*    Updated    : 10-JAN-94 by J. Doggett,                                    *
*       Modified #if def to abort if we're not on a known operating system.   *
*    Redelivered: 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
extern int
writehhh(h,image,headfile)
/*Header *h;*/
HEADER h;
short image[];
char *headfile;
{
char *datafile, *p;
int i, npix;
QFILE *ihead, *idata;

    /*
     * construct filename for data (hhd) file
     */
    datafile = malloc(strlen(headfile)+1);
    strcpy(datafile, headfile);

/******************
* writehhh page 2 *
******************/

    p = strrchr(datafile, 'h');
    *p++ = 'd';

    /*
     * the 'h' must either be the last character or be followed by
     * a semicolon (to be compatible with OpenVMS)
     */
    if (*p != '\0' && *p != ';') {
        fprintf(stderr, "Error: header filename must end with 'h'\n");
        free(datafile);
        return(-1);
    }
    /*
     * check to see whether header or data files already exist
     * (Don't bother under OpenVMS, just make a new version)
     */

    if (access(headfile,0) == 0) {
#if defined __VMS
/* We'll just make a new version */
#elif defined _UNIX
         fprintf(stderr, "Error: header file %s already exists\n",headfile);
         free(datafile);
         return(-1);
#else
         fprintf(stderr,"Unknown file system error in writehhh");
         exit(EXIT_FAILURE);
#endif
    }

    if (access(datafile,0) == 0) {
#if defined __VMS
/* We'll just make a new version */
#elif defined _UNIX
        fprintf(stderr, "Error: data file %s already exists\n",datafile);
        free(datafile);
        return(-1);
#else
        fprintf(stderr,"Unknown file system error in writehhh");
        exit(EXIT_FAILURE);
#endif
    }

    /*
     * write header file: 80 byte records, CR carriage control
     */
    ihead = qcreat(headfile,80,1);
    for (i=0; i < h->nhlines; i++) qwrite(ihead,h->hlines[i],80);
    qclose(ihead);
    /*
     * write data file: 512 byte records, no carriage control
     */
    idata = qcreat(datafile,512,0);
    npix = h->section_x_length * h->section_y_length;
    qwrite(idata, image, sizeof(short)*npix);
    qclose(idata);
    free(datafile);
    return(0);

} /* writehhh */
